@@ -1580,10 +1580,9 @@ class SQLCli(object):
                 for m_nPos in range(0, len(m_iter)):
                     if columntypes[m_nPos] == str:
                         # 字符串左对齐
-                        m_TableContentLine = m_TableContentLine + ' ' + \
-                                             str(m_iter[m_nPos])\
-                                                 .ljust(m_ColumnLength[m_nPos] - wide_chars(m_iter[m_nPos])) + \
-                                             ' |'
+                        m_TableContentLine = \
+                            m_TableContentLine + ' ' + \
+                            str(m_iter[m_nPos]).ljust(m_ColumnLength[m_nPos] - wide_chars(m_iter[m_nPos])) + ' |'
                     else:
                         # 数值类型右对齐
                         m_TableContentLine = m_TableContentLine + ' ' + \
